==> fbthrift-2024.11.04.00.gh.tar.gz: fbthrift-2024.11.04.00/thrift/lib/go/thrift/rocket_server_transport.go <==
/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package thrift

import (
	"context"
	"crypto/tls"
	"errors"
	"fmt"
	"io"
	"net"
	"runtime/debug"

	"github.com/facebook/fbthrift/thrift/lib/go/thrift/types"
	"github.com/rsocket/rsocket-go/core/transport"
)

type rocketServerTransport struct {
	listener    net.Listener
	processor   Processor
	acceptor    transport.ServerTransportAcceptor
	transportID TransportID
	log         func(format string, args ...interface{})
	connContext ConnContextFunc
}

func newRocketServerTransport(listener net.Listener, connContext ConnContextFunc, processor Processor, transportID TransportID, log func(format string, args ...interface{})) transport.ServerTransport {
	return &rocketServerTransport{
		listener:    listener,
		processor:   processor,
		log:         log,
		transportID: transportID,
		connContext: connContext,
	}
}

// Accept register incoming connection handler.
func (r *rocketServerTransport) Accept(acceptor transport.ServerTransportAcceptor) {
	r.acceptor = acceptor
}

// Listen listens on the network address addr and handles requests on incoming connections.
// You can specify notifier chan, it'll be sent true/false when server listening success/failed.
func (r *rocketServerTransport) Listen(ctx context.Context, notifier chan<- bool) error {
	notifier <- true
	go func() {
		<-ctx.Done()
		r.listener.Close()
	}()
	err := r.acceptLoop(ctx)
	if ctx.Err() != nil {
		return ctx.Err()
	}
	return err
}

// acceptLoop takes a context that will be decorated with ConnInfo and passed down to new clients.
func (r *rocketServerTransport) acceptLoop(ctx context.Context) error {
	for {
		conn, err := r.listener.Accept()
		if err != nil {
			if errors.Is(err, io.EOF) {
				return nil
			}
			select {
			case <-ctx.Done():
				return nil
			default:
				return fmt.Errorf("listener.Accept failed in rocketServerTransport.acceptLoop: %w", err)
			}
		}
		if conn == nil {
			continue
		}

		go func(ctx context.Context, conn net.Conn) {
			// Explicitly force TLS handshake protocol to run (if this is a TLS connection).
			//
			// Usually, TLS handshake is done implicitly/seamlessly by 'crypto/tls' package,
			// whenever Read/Write functions are invoked on a connection for the first time.
			// However, in our case, we require the handshake to be complete ahead of any
			// Read/Write calls - so that we can access ALPN value and choose the transport.
			tlsConn, isTLS := conn.(*tls.Conn)
			if isTLS {
				err = tlsConn.HandshakeContext(ctx)
				if err != nil {
					r.log("thrift: error performing TLS handshake with %s: %s\n", conn.RemoteAddr(), err)
					// Handshake failed, we cannot proceed with this connection - close it and return.
					tlsConn.Close()
					return
				}
			}

			ctx = r.connContext(ctx, conn)
			r.processRequests(ctx, conn)
		}(ctx, conn)
	}
}

func (r *rocketServerTransport) Close() (err error) {
	return r.listener.Close()
}

func (r *rocketServerTransport) processRequests(ctx context.Context, conn net.Conn) {
	connTransport := r.transportID

	// Use Rocket protocol right away if the server is running
	// in "UpgradeToRocket" mode and ALPN value is set to "rs".
	if r.transportID == TransportIDUpgradeToRocket {
		if connInfo, ok := ConnInfoFromContext(ctx); ok {
			tlsConnState := connInfo.TLS()
			if tlsConnState != nil && tlsConnState.NegotiatedProtocol == "rs" {
				connTransport = TransportIDRocket
			}
		}
	}

	switch connTransport {
	case TransportIDRocket:
		r.processRocketRequests(ctx, conn)
	case TransportIDUpgradeToRocket:
		processor := newRocketUpgradeProcessor(r.processor)
		headerProtocol, err := NewHeaderProtocol(conn)
		if err != nil {
			r.log("thrift: error constructing header protocol from %s: %s\n", conn.RemoteAddr(), err)
			return
		}
		if err := r.processHeaderRequest(ctx, headerProtocol, processor); err != nil {
			r.log("thrift: error processing first header request from %s: %s\n", conn.RemoteAddr(), err)
			return
		}
		if processor.upgraded {
			r.processRocketRequests(ctx, conn)
		} else {
			if err := r.processHeaderRequests(ctx, headerProtocol, processor); err != nil {
				r.log("thrift: error processing additional header request from %s: %s\n", conn.RemoteAddr(), err)
			}
		}
	case TransportIDHeader:
		headerProtocol, err := NewHeaderProtocol(conn)
		if err != nil {
			r.log("thrift: error constructing header protocol from %s: %s\n", conn.RemoteAddr(), err)
			return
		}
		if err := r.processHeaderRequests(ctx, headerProtocol, r.processor); err != nil {
			r.log("thrift: error processing header request from %s: %s\n", conn.RemoteAddr(), err)
		}
	}
}

func (r *rocketServerTransport) processRocketRequests(ctx context.Context, conn net.Conn) {
	r.acceptor(ctx, transport.NewTransport(transport.NewTCPConn(conn)), func(*transport.Transport) {})
}

func (r *rocketServerTransport) processHeaderRequest(ctx context.Context, protocol types.Protocol, processor Processor) error {
	exc := process(ctx, processor, protocol)
	if isEOF(exc) {
		return exc
	}
	if exc != nil {
		protocol.Flush()
		return exc
	}
	return nil
}

func (r *rocketServerTransport) processHeaderRequests(ctx context.Context, protocol types.Protocol, processor Processor) error {
	defer func() {
		if err := recover(); err != nil {
			r.log("panic in processor: %v: %s", err, debug.Stack())
		}
	}()
	defer protocol.Close()
	var err error
	for err == nil {
		err = r.processHeaderRequest(ctx, protocol, processor)
	}
	if isEOF(err) {
		return nil
	}
	protocol.Flush()
	// graceful exit.  client closed connection
	return err
}

==> fbthrift-2025.04.14.00.gh.tar.gz: fbthrift-2025.04.14.00/thrift/lib/go/thrift/rocket_server_transport.go <==
/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package thrift

import (
	"context"
	"crypto/tls"
	"errors"
	"fmt"
	"io"
	"net"
	"runtime/debug"

	"github.com/facebook/fbthrift/thrift/lib/go/thrift/stats"
	"github.com/facebook/fbthrift/thrift/lib/thrift/rocket_upgrade"
	"github.com/rsocket/rsocket-go/core/transport"
)

type rocketServerTransport struct {
	listener    net.Listener
	processor   Processor
	acceptor    transport.ServerTransportAcceptor
	transportID TransportID
	connContext ConnContextFunc
	log         func(format string, args ...any)
	stats       *stats.ServerStats
	pstats      map[string]*stats.TimingSeries
}

func newRocketServerTransport(
	listener net.Listener,
	connContext ConnContextFunc,
	processor Processor,
	transportID TransportID,
	log func(format string, args ...any),
	stats *stats.ServerStats,
	pstats map[string]*stats.TimingSeries,
) transport.ServerTransport {
	return &rocketServerTransport{
		listener:    listener,
		processor:   processor,
		transportID: transportID,
		connContext: connContext,
		log:         log,
		stats:       stats,
		pstats:      pstats,
	}
}

// Accept register incoming connection handler.
func (r *rocketServerTransport) Accept(acceptor transport.ServerTransportAcceptor) {
	r.acceptor = acceptor
}

// Listen listens on the network address addr and handles requests on incoming connections.
// You can specify notifier chan, it'll be sent true/false when server listening success/failed.
func (r *rocketServerTransport) Listen(ctx context.Context, notifier chan<- bool) error {
	notifier <- true
	go func() {
		<-ctx.Done()
		r.listener.Close()
	}()
	err := r.acceptLoop(ctx)
	if ctx.Err() != nil {
		return ctx.Err()
	}
	return err
}

// acceptLoop takes a context that will be decorated with ConnInfo and passed down to new clients.
func (r *rocketServerTransport) acceptLoop(ctx context.Context) error {
	for {
		conn, err := r.listener.Accept()
		if err != nil {
			if errors.Is(err, io.EOF) {
				return nil
			}
			select {
			case <-ctx.Done():
				return nil
			default:
				return fmt.Errorf("listener.Accept failed in rocketServerTransport.acceptLoop: %w", err)
			}
		}
		if conn == nil {
			continue
		}

		go func(ctx context.Context, conn net.Conn) {
			// Explicitly force TLS handshake protocol to run (if this is a TLS connection).
			//
			// Usually, TLS handshake is done implicitly/seamlessly by 'crypto/tls' package,
			// whenever Read/Write functions are invoked on a connection for the first time.
			// However, in our case, we require the handshake to be complete ahead of any
			// Read/Write calls - so that we can access ALPN value and choose the transport.
			tlsConn, isTLS := conn.(*tls.Conn)
			if isTLS {
				err = tlsConn.HandshakeContext(ctx)
				if err != nil {
					r.log("thrift: error performing TLS handshake with %s: %s\n", conn.RemoteAddr(), err)
					// Handshake failed, we cannot proceed with this connection - close it and return.
					tlsConn.Close()
					return
				}
			}

			ctxConn := r.connContext(ctx, conn)
			r.processRequests(ctxConn, conn)
		}(ctx, conn)
	}
}

func (r *rocketServerTransport) Close() error {
	return r.listener.Close()
}

func (r *rocketServerTransport) processRequests(ctx context.Context, conn net.Conn) {
	// keep track of the number of connections established and closed
	// over time
	r.stats.ConnsEstablished.RecordEvent()
	// update current connection count
	r.stats.ConnCount.Incr()
	defer func() {
		r.stats.ConnsClosed.RecordEvent()
		r.stats.ConnCount.Decr()
	}()

	connTransport := r.transportID
	// Use Rocket protocol right away if the server is running
	// in "UpgradeToRocket" mode and ALPN value is set to "rs".
	if r.transportID == TransportIDUpgradeToRocket {
		if connInfo, ok := ConnInfoFromContext(ctx); ok {
			tlsConnState := connInfo.TLS()
			if tlsConnState != nil && tlsConnState.NegotiatedProtocol == "rs" {
				connTransport = TransportIDRocket
			}
		}
	}

	switch connTransport {
	case TransportIDRocket:
		r.processRocketRequests(ctx, conn)
	case TransportIDUpgradeToRocket:
		ruHandler := newRocketUpgradeHandler()
		ruProcessor := rocket_upgrade.NewRocketUpgradeProcessor(ruHandler)
		ruCompProcessor := NewCompositeProcessor()
		ruCompProcessor.Include(r.processor)
		ruCompProcessor.Include(ruProcessor)

		headerProtocol, err := NewHeaderProtocol(conn)
		if err != nil {
			r.log("thrift: error constructing header protocol from %s: %s\n", conn.RemoteAddr(), err)
			return
		}
		if err := r.processHeaderRequest(ctx, headerProtocol, ruCompProcessor); err != nil {
			r.log("thrift: error processing first header request from %s: %s\n", conn.RemoteAddr(), err)
			return
		}
		if ruHandler.upgradeInvoked {
			r.processRocketRequests(ctx, conn)
		} else {
			if err := r.processHeaderRequests(ctx, headerProtocol, ruCompProcessor); err != nil {
				r.log("thrift: error processing additional header request from %s: %s\n", conn.RemoteAddr(), err)
			}
		}
	case TransportIDHeader:
		headerProtocol, err := NewHeaderProtocol(conn)
		if err != nil {
			r.log("thrift: error constructing header protocol from %s: %s\n", conn.RemoteAddr(), err)
			return
		}
		if err := r.processHeaderRequests(ctx, headerProtocol, r.processor); err != nil {
			r.log("thrift: error processing header request from %s: %s\n", conn.RemoteAddr(), err)
		}
	}
}

func (r *rocketServerTransport) processRocketRequests(ctx context.Context, conn net.Conn) {
	r.acceptor(ctx, transport.NewTransport(transport.NewTCPConn(conn)), func(*transport.Transport) {})
}

func (r *rocketServerTransport) processHeaderRequest(ctx context.Context, protocol Protocol, processor Processor) error {
	exc := process(ctx, processor, protocol, r.pstats)
	if isEOF(exc) {
		return exc
	}
	if exc != nil {
		protocol.Flush()
		return exc
	}
	return nil
}

func (r *rocketServerTransport) processHeaderRequests(ctx context.Context, protocol Protocol, processor Processor) error {
	defer func() {
		if err := recover(); err != nil {
			r.log("panic in processor: %v: %s", err, debug.Stack())
		}
	}()
	defer protocol.Close()
	var err error
	for err == nil {
		err = r.processHeaderRequest(ctx, protocol, processor)
	}
	if isEOF(err) {
		return nil
	}
	protocol.Flush()
	// graceful exit.  client closed connection
	return err
}
